require 'spec_helper'

describe SmartFilter do
  before(:each) do
    stub_filter
  end

  describe '.analyze' do
    subject { SmartFilter.new }

    before(:each) do
      subject.analyze('some text')
    end

    it { subject.instance_variable_get(:@input).should_not be_nil }
    it { subject.instance_variable_get(:@stats).should_not be_nil }
    it { subject.instance_variable_get(:@output).should_not be_nil }
    it { subject.instance_variable_get(:@result).should_not be_nil }
    it { subject.instance_variable_get(:@result).should be_a(Stoolie::Result) }
  end

  describe 'with benign text' do
    subject { SmartFilter.new }

    before(:each) do
      stub_filter
      subject.analyze('benign text')
    end

    it { subject.should_not be_is_insecure }
    it { subject.should_not be_is_spam }
    it { subject.should_not be_is_offensive }
  end

  describe '.is_insecure? with malicious text exceeding the javascript threshold' do
    subject { SmartFilter.new }

    context 'because of tags' do
      before(:each) do
        stub_filter('javascript_tags' => 1)
        subject.analyze('<script>malicious text</script>')
      end

      it { subject.should be_is_insecure }
    end

    context 'because of protocols' do
      before(:each) do
        stub_filter('javascript_protocols' => 1)
        subject.analyze('<script>malicious text</script>')
      end

      it { subject.should be_is_insecure }
    end

    context 'because of attributes' do
      before(:each) do
        stub_filter('javascript_attributes' => 1)
        subject.analyze('<script>malicious text</script>')
      end

      it { subject.should be_is_insecure }
    end
  end

  describe '.is_spam? with malicious text exceeding the spam threshold' do
    subject { SmartFilter.new }

    context 'because of link density' do
      before(:each) do
        stub_filter('prevoty_link_density' => {'http://test.com' => 2, 'http://alsotest.com' => 1})
        subject.analyze('spammy text')
      end

      it { subject.should be_is_spam }
    end

    context 'because of spam features' do
      before(:each) do
        stub_filter('prevoty_spam_features' => 2)
        subject.analyze('spammy text')
      end

      it { subject.should be_is_spam }
    end
  end

  describe '.is_blacklisted? with malicious text exceeding the blacklist threshold' do
    subject { SmartFilter.new }

    context 'because of blacklisted phrases' do
      before(:each) do
        stub_filter('blacklisted_phrases' => 1)
        subject.analyze('offensive text')
      end

      it { subject.should be_is_blacklisted }
    end
  end

  describe '.is_offensive? with malicious text exceeding the offensive threshold' do
    subject { SmartFilter.new }

    context 'because of flagged phrases' do
      before(:each) do
        stub_filter('flagged_phrases' => 6)
        subject.analyze('offensive text')
      end

      it { subject.should be_is_offensive }
    end

    context 'because of profanity' do
      before(:each) do
        stub_filter('prevoty_profanity_features' => 3)
        subject.analyze('offensive text')
      end

      it { subject.should be_is_offensive }
    end
  end

end
